using ProjectElectricLocomotive;
using ProjectElectricLocomotive.DrawingObjects;
using ProjectElectricLocomotive.MovementStrategy;
using System;

namespace ElectricLocomotive
{
    public partial class ElectricLocomotive : Form
    {

        private DrawingLocomotive? _drawingLocomotive;
       
        private AbstractStrategy? _abstractStrategy;

        public ElectricLocomotive()
        {
            InitializeComponent();
        }

        private void Draw()
        {
            if (_drawingLocomotive == null)
            {
                return;
            }
            Bitmap bmp = new(pictureBoxElectricLocomotive.Width, pictureBoxElectricLocomotive.Height);
            Graphics gr = Graphics.FromImage(bmp);
            _drawingLocomotive.DrawTransport(gr);
            pictureBoxElectricLocomotive.Image = bmp;
        }

        private void buttonCreateElectricLocomotive_Click(object sender, EventArgs e)
        {
            Random random = new Random();
            _drawingLocomotive = new DrawingElectricLocomotive(random.Next(100, 300),random.Next(1000, 3000),
            Color.FromArgb(random.Next(0, 256), random.Next(0, 256), random.Next(0, 256)),
            Color.FromArgb(random.Next(0, 256), random.Next(0, 256), random.Next(0, 256)),
            Convert.ToBoolean(random.Next(0, 2)), Convert.ToBoolean(random.Next(0, 2)), 
            pictureBoxElectricLocomotive.Width, pictureBoxElectricLocomotive.Height);
            _drawingLocomotive.SetPosition(random.Next(10, 100), random.Next(10, 100));
            Draw();
        }

        private void buttonCreateLocomotive_Click(object sender, EventArgs e)
        {
            Random rnd = new Random();
            _drawingLocomotive = new DrawingLocomotive(rnd.Next(100, 300), rnd.Next(1000, 3000),
                Color.FromArgb(rnd.Next(0, 256), rnd.Next(0, 256), rnd.Next(0, 256)),
                pictureBoxElectricLocomotive.Width, pictureBoxElectricLocomotive.Height);
            _drawingLocomotive.SetPosition(rnd.Next(10, 100), rnd.Next(10, 100));
            Draw();
        }

        private void buttonMove_Click(object sender, EventArgs e)
        {
            if (_drawingLocomotive == null)
            {
                return;
            }
            string name = ((Button)sender)?.Name ?? string.Empty;
            switch (name)
            {
                case "buttonUp":
                    _drawingLocomotive.MoveTransport(DirectionType.Up);
                    break;
                case "buttonDown":
                    _drawingLocomotive.MoveTransport(DirectionType.Down);
                    break;
                case "buttonLeft":
                    _drawingLocomotive.MoveTransport(DirectionType.Left);
                    break;
                case "buttonRight":
                    _drawingLocomotive.MoveTransport(DirectionType.Right);
                    break;
            }
            Draw();
        }

        private void buttonStep_Click(object sender, EventArgs e)
        {
            if (_drawingLocomotive == null)
            {
                return;
            }
            if (comboBoxStrategy.Enabled)
            {
                _abstractStrategy = comboBoxStrategy.SelectedIndex switch
                {
                    0 => new MoveToCenter(),
                    1 => new MoveToRightCorner(),
                    _ => null,
                };
                if (_abstractStrategy == null)
                {
                    return;
                }
                _abstractStrategy.SetData(new
                DrawingObjectLocomotive(_drawingLocomotive), pictureBoxElectricLocomotive.Width,
                pictureBoxElectricLocomotive.Height);
                comboBoxStrategy.Enabled = false;
            }
            if (_abstractStrategy == null)
            {
                return;
            }
            _abstractStrategy.MakeStep();
            Draw();
            if (_abstractStrategy.GetStatus() == Status.Finish)
            {
                comboBoxStrategy.Enabled = true;
                _abstractStrategy = null;
            }
        }
    }
}